import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:labs/components/extensions/context_x.dart';
import 'package:labs/presentation/details_page/details_page.dart';
import 'package:labs/presentation/home_page/bloc/events.dart';
import 'package:labs/presentation/like_bloc/like_event.dart';
import 'package:labs/presentation/like_bloc/like_state.dart';
import 'package:labs/repo/potter_repo.dart';
import '../../components/utils/debounce.dart';
import '../../domain/models/Student.dart';
import '../common/svg_objects.dart';
import '../like_bloc/like_bloc.dart';
import '../locale_bloc/locale_bloc.dart';
import '../locale_bloc/locale_events.dart';
import '../locale_bloc/locale_state.dart';
import 'bloc/bloc.dart';
import 'bloc/state.dart';
part 'card.dart';

class MyHomePage extends StatefulWidget {
  const MyHomePage({super.key});

  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  @override
  Widget build(BuildContext context) {
    return const Scaffold(body: _Body());
  }
}

class _Body extends StatefulWidget {
  const _Body();

  @override
  State<_Body> createState() => _BodyState();
}

class _BodyState extends State<_Body> {
  final searchController = TextEditingController();
  final scrollController = ScrollController();
  final PotterRepo repo = PotterRepo();
  late Future<List<Student>?> data;

  @override
  void initState() {
    SvgObjects.init();

    WidgetsBinding.instance.addPostFrameCallback((_) {
      context.read<HomeBloc>().add(const HomeLoadDataEvent());
      context.read<LikeBloc>().add(const LoadLikesEvent());
    });

    scrollController.addListener(_onNextPageListener);

    super.initState();
  }

  void _onNextPageListener() {
    if (scrollController.offset >= scrollController.position.maxScrollExtent) {
      final bloc = context.read<HomeBloc>();
      if (!bloc.state.isPaginationLoading) {
        bloc.add(HomeLoadDataEvent(
          search: searchController.text,
          nextPage: bloc.state.data?.nextPage,
        ));
      }
    }
  }

  void _onLike(String? id, String title, bool isLiked) {
    if (id != null) {
      context.read<LikeBloc>().add(ChangeLikeEvent(id));
      _showSnackBar(context, title, !isLiked);
    }
  }

  @override
  void dispose() {
    searchController.dispose();
    scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Padding(
        padding: EdgeInsets.only(top: MediaQuery.of(context).padding.top),
        child: Column(
          children: [
            Padding(
              padding: const EdgeInsets.all(12),
              child: CupertinoSearchTextField(
                controller: searchController,
                placeholder: context.locale.search,
                onChanged: (search) {
                  Debounce.run(
                      () => context.read<HomeBloc>().add(HomeLoadDataEvent(search: search)));
                },
              ),
            ),
            GestureDetector(
              onTap: () => context.read<LocaleBloc>().add(const ChangeLocaleEvent()),
              child: SizedBox.square(
                dimension: 50,
                child: Padding(
                  padding: const EdgeInsets.only(right: 12),
                  child: BlocBuilder<LocaleBloc, LocaleState>(
                    builder: (context, state) {
                      return state.currentLocale.languageCode == 'ru'
                          ? const SvgRu()
                          : const SvgUk();
                    },
                  ),
                ),
              ),
            ),
            BlocBuilder<HomeBloc, HomeState>(
              builder: (context, state) => state.error != null
                  ? Text(
                      state.error ?? '',
                      style: Theme.of(context).textTheme.headlineSmall?.copyWith(color: Colors.red),
                    )
                  : state.isLoading
                      ? const CircularProgressIndicator()
                      : BlocBuilder<LikeBloc, LikeState>(
                          builder: (context, likeState) {
                            return Expanded(
                              child: RefreshIndicator(
                                onRefresh: _onRefresh,
                                child: ListView.builder(
                                  controller: scrollController,
                                  padding: EdgeInsets.zero,
                                  itemCount: state.data?.data?.length ?? 0,
                                  itemBuilder: (context, index) {
                                    final data = state.data?.data?[index];
                                    return data != null
                                        ? _Card.fromData(
                                            data,
                                            onLike: _onLike,
                                            isLiked: likeState.likedIds?.contains(data.id) == true,
                                            onTap: () => _navToDetails(context, data),
                                          )
                                        : const SizedBox.shrink();
                                  },
                                ),
                              ),
                            );
                          },
                        ),
            ),
            BlocBuilder<HomeBloc, HomeState>(
              builder: (context, state) => state.isPaginationLoading
                  ? const CircularProgressIndicator()
                  : const SizedBox.shrink(),
            )
          ],
        ),
      ),
    );
  }

  void _navToDetails(BuildContext context, Student data) {
    Navigator.push(
      context,
      CupertinoPageRoute(builder: (context) => DetailsPage(data)),
    );
  }

  Future<void> _onRefresh() {
    context.read<HomeBloc>().add(HomeLoadDataEvent(search: searchController.text));
    return Future.value(null);
  }

  void _showSnackBar(BuildContext context, String title, bool isLiked) {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text(
          '${context.locale.studentWord} $title ${isLiked ? context.locale.liked : context.locale.disliked}',
          style: Theme.of(context).textTheme.bodyLarge,
        ),
        backgroundColor: Colors.lightBlueAccent,
        duration: const Duration(seconds: 1),
      ));
    });
  }
}
